﻿using System;
using System.Linq;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Collections; //neede for array
using System.Reflection; //needed for assembly
using System.Drawing.Imaging; //needed for imageattributes

namespace SmartDeviceTest
{
    

    public partial class OwnerDrawnList : Control
    {
        int scrollWidth;
        int itemHeight = -1;
        int selectedIndex = -1;
        Bitmap offScreen;
        VScrollBar vs;
        ArrayList items;

        public OwnerDrawnList()
        {
            this.vs = new VScrollBar();
            if (this.vs.Width > 240)
                scrollWidth = this.vs.Width * 2;
            else
                scrollWidth = this.vs.Width;
            this.vs.Parent = this;
            this.vs.Visible = false;
            this.vs.SmallChange = 1;
            this.vs.ValueChanged += new EventHandler(this.ScrollValueChanged);
            // Items to draw
            this.items = new ArrayList();
        }

        public ArrayList Items
        {
            get { return this.items; }
        }

        protected Bitmap OffScreen
        {
            get { return this.offScreen; }
        }

        protected VScrollBar VScrollBar
        {
            get { return this.vs; }
        }

        public event EventHandler SelectedIndexChanged;

        // Raise the SelectedIndexChanged event
        protected virtual void OnSelectedIndexChanged(EventArgs e)
        {
            if (this.SelectedIndexChanged != null)
                this.SelectedIndexChanged(this, e);
        }

        public int SelectedIndex
        {
            get { return this.selectedIndex; }
            set
            {
                this.selectedIndex = value;
                if (this.SelectedIndexChanged != null)
                    this.SelectedIndexChanged(this, EventArgs.Empty);
            }
        }

        protected void ScrollValueChanged(object o, EventArgs e)
        {
            this.Refresh();
        }

        protected virtual int ItemHeight
        {
            get { return this.itemHeight; }
            set { this.itemHeight = value; }
        }

        // If the requested index is before the first visible index then set the
        // first item to be the requested index. If it is after the last visible
        // index, then set the last visible index to be the requested index.
        public void EnsureVisible(int index)
        {
            if (index < this.vs.Value)
            {
                this.vs.Value = index;
                this.Refresh();
            }
            else if (index >= this.vs.Value + this.DrawCount)
            {
                this.vs.Value = index - this.DrawCount + 1;
                this.Refresh();
            }
        }
        
        //on key press event
        protected override void OnKeyDown(KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Down:
                    if (this.SelectedIndex < this.vs.Maximum)
                    {
                        EnsureVisible(++this.SelectedIndex);
                        this.Refresh();
                    }
                    break;
                case Keys.Up:
                    if (this.SelectedIndex > this.vs.Minimum)
                    {
                        EnsureVisible(--this.SelectedIndex);
                        this.Refresh();
                    }
                    break;
            }

            base.OnKeyDown(e);
        }

        protected int DrawCount
        {
            get
            {
                if (this.vs.Value + this.vs.LargeChange > this.vs.Maximum)
                    return this.vs.Maximum - this.vs.Value;
                else
                    return this.vs.LargeChange;
            }
        }

        protected override void OnResize(EventArgs e)
        {
            int viewableItemCount = this.ClientSize.Height / this.ItemHeight;
            this.vs.Bounds = new Rectangle(this.ClientSize.Width - scrollWidth,
                0,
                scrollWidth,
                this.ClientSize.Height);


            if (this.items.Count > viewableItemCount)
            {
                this.vs.Visible = true;
                this.vs.LargeChange = viewableItemCount;

                this.offScreen = new Bitmap(this.ClientSize.Width - scrollWidth, this.ClientSize.Height);
            }
            else
            {
                this.vs.Visible = false;
                this.vs.LargeChange = this.items.Count;

                this.offScreen = new Bitmap(this.ClientSize.Width, this.ClientSize.Height);
            }

            this.vs.Maximum = this.items.Count - 1;
        }

        protected Color CalcTextColor(Color backgroundColor)
        {
            if (backgroundColor.Equals(Color.Empty))
                return Color.Black;

            int sum = backgroundColor.R + backgroundColor.G + backgroundColor.B;

            if (sum > 256)
                return Color.Black;
            else
                return Color.White;
        }
    }

    public class MyListView : OwnerDrawnList
    {
        const int Column1Left = 5;
        const int Column2Left = 25;

        public MyListView()
        {
            // We need a total of 5 rows, so the height is 180/5=36
            Graphics g = this.CreateGraphics();
            Font font = new Font(this.Font.Name, 10, FontStyle.Bold);
            // Calc line height to be height of letter A plus 10%
            int fontHeight = (int)(g.MeasureString("A", font).Height * 1.1);
            this.ItemHeight = fontHeight * 2;
            g.Dispose();
        }
              
            
            

        protected override void OnKeyDown(KeyEventArgs e)
        {
            switch (e.KeyCode)
            {
                case Keys.Return:
                    MessageBox.Show("You selected item " + this.SelectedIndex.ToString(),
                         "Item selected",
                         MessageBoxButtons.OK,
                         MessageBoxIcon.Asterisk,
                         MessageBoxDefaultButton.Button1);
                    break;
            }

            base.OnKeyDown(e);
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            // Declare vars
            Font font;
            Color fontColor;
            string bmpName;

            Graphics gOffScreen = Graphics.FromImage(this.OffScreen);
            gOffScreen.FillRectangle(new SolidBrush(this.BackColor), this.ClientRectangle);
            int itemTop = 0;

            for (int n = this.VScrollBar.Value; n <= this.VScrollBar.Value + DrawCount; n++)
            {
                if (n == this.SelectedIndex) //check whether if mail item is selected if so highlight the background color
                {
                    //paint the selected rectangle with system background color
                    gOffScreen.FillRectangle(new SolidBrush(SystemColors.Highlight),
                         0,
                         itemTop,
                         this.ClientSize.Width - (this.VScrollBar.Visible ? this.VScrollBar.Width : 0),
                         this.ItemHeight);
                    fontColor = CalcTextColor(SystemColors.Highlight);
                }
                else
                    fontColor = this.ForeColor;

                gOffScreen.DrawLine(new Pen(Color.DarkGray),
                     1,
                     itemTop + this.ItemHeight,
                     this.ClientSize.Width - (this.VScrollBar.Visible ? this.VScrollBar.Width : 2),
                     itemTop + this.ItemHeight);

                MailItem lvi = (MailItem)this.Items[n];//paint appropriate mail item to background

                if (lvi.icon == MailIcon.unopened)
                {
                    font = new Font(this.Font.Name, 10, FontStyle.Bold);
                    bmpName = "Unread.bmp";
                }
                else
                {
                    font = new Font(this.Font.Name, 10, FontStyle.Regular);
                    bmpName = "Read.bmp";
                }


                Bitmap bmp = new Bitmap(SmartDeviceTest.Properties.Resources.Read);
                Bitmap bmp2 = new Bitmap(SmartDeviceTest.Properties.Resources.Unread);
                // Bitmap bmp = null;

                // To draw a transparent image, we need to set the transparent           
                ImageAttributes ia = new ImageAttributes();
                ia.SetColorKey(Color.White, Color.White);

                // Set the image rectangle
                Rectangle imgRect = new Rectangle(Column1Left, itemTop + 3, bmp.Width, bmp.Height);

                // Draw the image
                if (lvi.icon == MailIcon.opened)
                    gOffScreen.DrawImage(bmp, imgRect, 0, 0, bmp.Width, bmp.Height, GraphicsUnit.Pixel, ia);
                else
                    gOffScreen.DrawImage(bmp2, imgRect, 0, 0, bmp.Width, bmp.Height, GraphicsUnit.Pixel, ia);
                // Draw the mail sender
                gOffScreen.DrawString(lvi.sender, font, new SolidBrush(fontColor), Column2Left, itemTop);
                // Draw the mail subject
                gOffScreen.DrawString(lvi.subject, font, new SolidBrush(fontColor), Column2Left, itemTop + (ItemHeight / 2));

                font.Dispose();
                bmp.Dispose();

                itemTop += this.ItemHeight;
            }

            e.Graphics.DrawImage(this.OffScreen, 0, 0);

            gOffScreen.Dispose();
        }

        // Draws the external border around the control.   
        protected override void OnPaintBackground(PaintEventArgs e)
        {
            // e.Graphics.DrawRectangle(new Pen(Color.Black), 0, 0, this.ClientSize.Width - 1, this.ClientSize.Height - 1);
        }
    }
    enum MailIcon
    {
        unopened, opened
    }

    class MailItem
    {
        public MailIcon icon;
        public string sender;
        public string subject;

        public MailItem(MailIcon Icon, string Sender, string Subject)
        {
            this.icon = Icon;
            this.sender = Sender;
            this.subject = Subject;
        }
    }
}
